#include <stdio.h>
#include<stdlib.h>
#include<math.h>
#include"SimpleImage.h"
#include"ppmio.h"
#include"otsu.h"
#include"thinning.h"

double SobelX[3][3] = {{-1.0,0.0,1.0},{-2.0,0.0,2.0},{-1.0,0.0,1.0}};
double SobelY[3][3] = {{-1.0,-2.0,-1.0},{0.0,0.0,0.0},{1.0,2.0,1.0}};
//double SobelX[3][3] = {{0.0,0.0,0.0},{0.0,-1.0,1.0},{0.0,0.0,0.0}};
//double SobelY[3][3] = {{0.0,0.0,0.0},{0.0,-1.0,0.0},{0.0,1.0,0.0}};

double getVal0to255(double val){
  int vali =((int)(((double)(255.0*val))));
  if(fabs((double)(val-vali))>=0.5)vali++;
  if(vali<0)vali=0;
  if(vali>255)vali=255;
  return ((double)(vali));
}

void setGrad(Image *in,Image *out){
  int i,j;
  double max=0.0;
  double min=0.0;
  bool firstcheck=true;
  for(i=0;i<in->sy;i++){
    for(j=0;j<in->sx;j++){
      int ii,jj;
      out->img[i][j]=0.0;
      double dx = 0.0;
      double dy = 0.0;
      for(ii=-1;ii<=1;ii++){
	int i2 = i+ii;if(i2<0)i2=0;if(i2>in->sy-1)i2=in->sy-1;
	for(jj=-1;jj<=1;jj++){
	  int j2 = j+jj;if(j2<0)j2=0;if(j2>in->sx-1)j2=in->sx-1;
	  dx += in->img[i2][j2]*SobelX[(ii+1)][(jj+1)];
	  dy += in->img[i2][j2]*SobelY[(ii+1)][(jj+1)];
	  
	  
	}
      }
      out->img[i][j]= (sqrt(dx*dx+dy*dy));
      if(firstcheck){
	max=min=out->img[i][j];
	firstcheck=false;
      }else{
	if(max<out->img[i][j])max=out->img[i][j];
	if(min>out->img[i][j])min=out->img[i][j];
      }
    }
    
  }
  double range = fabs(max-min);
  if(range<=0.0)range=1.0;
 
  
  for(i=0;i<in->sy;i++)
    for(j=0;j<in->sx;j++){
      if(out->img[i][j]>=255.0)out->img[i][j]=255.0;
      out->img[i][j]=255.0-out->img[i][j];
    }

}

int main(int argc, char* argv[])
{
  
  int i,j;
  Image *inR = new Image();
  Image *inG = new Image();
  Image *inB = new Image();
  getPPM(inR,inG,inB,argv[1]);
  Image *outR = new Image(inR->sx,inR->sy);
  Image *outG = new Image(inG->sx,inG->sy);
  Image *outB = new Image(inB->sx,inB->sy);
  Image *gray = new Image(inR->sx,inR->sy);
  Image *grad = new Image(inR->sx,inR->sy);

  for(i=0;i<gray->sy;i++)
    for(j=0;j<gray->sx;j++){
      gray->img[i][j] = (inR->img[i][j]+inG->img[i][j]+inB->img[i][j])/3.0;
    }
  setGrad(gray,grad);
  savePPM(grad,grad,grad,argv[3]);
  /* main processing */
  

  int HISTSIZE=256;
  Otsu *myotsu = new Otsu(HISTSIZE);

  double threshold = ((double)(myotsu->find_otsu_threshold(grad)));
  printf("Otsu's threshold value = %lf\n",threshold);
  delete myotsu;
  double WhiteLABEL=255.0;
  for(i=0;i<grad->sy;i++)
    for(j=0;j<grad->sx;j++){
      if(grad->img[i][j]>=threshold){
	grad->img[i][j] = 0.0;
      }else{
	grad->img[i][j] = WhiteLABEL;
      }
    }
  //savePPM(grad,grad,grad,argv[4]);
  //thinning
  
  thinning(grad,WhiteLABEL);  

  /* end main processing */
for(i=0;i<outR->sy;i++)
    for(j=0;j<outR->sx;j++){
      outR->img[i][j]=outG->img[i][j]=outB->img[i][j]=grad->img[i][j];
    }

  
  savePPM(outR,outG,outB,argv[2]);
  delete grad;
  delete gray;
  delete inR;
  delete inG;
  delete inB;
  delete outR;
  delete outG;
  delete outB;
  
  return 0;
}
